#ifndef CGXCDSM_H
#define CGXCDSM_H


#ifdef __cplusplus
extern "C" {
#endif
#include <cgx.h>
  /*
    int createBlendedNurbs(int nr, Points **pntpntr, Lines *line, Lcmb *lcmb, Gsur *surf );
    double repairNurbs( Nurbs *nurbs, int nr, int deg, int dir);
    int evalNurbs( Nurbs *nurbs, int nr, int sum_p, double *pnt_u, double *pnt_v, Points *pnt);
    void projSurfToNurbs( Nurbs *nurbs, int nr, Gsur *surf, int snr, Nodes *node );
    void projPntsToNurbs( Nurbs *nurbs, int nr, Sets *set, int setNr, Points *pnt);
    int getNurbsType( Nurbs *nurbs, int nr, int *axis, int *sector );
    void trimNurbs( Nurbs *nurbs, int nr, int patch, double TOL_AMBIG);
  */
#ifdef __cplusplus
}
#endif


#endif
